@@ -2,43 +2,37 @@
 # this repository contains the full copyright notices and license terms.
 import os
 import unittest
-import doctest
-from itertools import chain
 from lxml import etree
 from decimal import Decimal
-from io import BytesIO
+from io import BytesIO, open
 
-from mock import Mock, patch
+try:
+    from unittest.mock import Mock, patch
+except ImportError:
+    from mock import Mock, patch
 
 import trytond.tests.test_tryton
-from trytond.tests.test_tryton import ModuleTestCase
-from trytond.tests.test_tryton import DB_NAME, USER, CONTEXT
+from trytond.tests.test_tryton import ModuleTestCase, with_transaction
 from trytond.transaction import Transaction
 from trytond.exceptions import UserError
 from trytond.modules.account_payment_sepa.payment import CAMT054
 from trytond.pool import Pool
 
+from trytond.modules.currency.tests import create_currency
+from trytond.modules.company.tests import create_company, set_company
+from trytond.modules.account.tests import create_chart
+
 
 def setup_environment():
     pool = Pool()
     Address = pool.get('party.address')
-    Company = pool.get('company.company')
-    Currency = pool.get('currency.currency')
     Party = pool.get('party.party')
     Bank = pool.get('bank')
 
-    company, = Company.search([
-            ('rec_name', '=', 'Dunder Mifflin'),
-            ])
-    euro, = Currency.create([{
-                'name': 'Euro',
-                'symbol': 'EUR',
-                'code': 'EUR',
-                }])
-    company.currency = euro
+    currency = create_currency('EUR')
+    company = create_company(currency=currency)
     company.party.sepa_creditor_identifier = 'BE68539007547034'
     company.party.save()
-    company.save()
     bank_party = Party(name='European Bank')
     bank_party.save()
     bank = Bank(party=bank_party, bic='BICODEBBXXX')
@@ -121,83 +115,87 @@ def validate_file(flavor, kind, xsd=None):
     company = environment['company']
     bank = environment['bank']
     customer = environment['customer']
-    company_account, customer_account = setup_accounts(
-        bank, company, customer)
-    setup_mandate(company, customer, customer_account)
-    journal = setup_journal(flavor, kind, company, company_account)
+    with set_company(company):
+        company_account, customer_account = setup_accounts(
+            bank, company, customer)
+        setup_mandate(company, customer, customer_account)
+        journal = setup_journal(flavor, kind, company, company_account)
 
-    payment, = Payment.create([{
-                'company': company,
-                'party': customer,
-                'journal': journal,
-                'kind': kind,
-                'amount': Decimal('1000.0'),
-                'state': 'approved',
-                'description': 'PAYMENT',
-                'date': Date.today(),
-                }])
+        payment, = Payment.create([{
+                    'company': company,
+                    'party': customer,
+                    'journal': journal,
+                    'kind': kind,
+                    'amount': Decimal('1000.0'),
+                    'state': 'approved',
+                    'description': 'PAYMENT',
+                    'date': Date.today(),
+                    }])
 
-    session_id, _, _ = ProcessPayment.create()
-    process_payment = ProcessPayment(session_id)
-    with Transaction().set_context(active_ids=[payment.id]):
-        _, data = process_payment.do_process(None)
-    group, = PaymentGroup.browse(data['res_id'])
-    message, = group.sepa_messages
-    assert message.type == 'out', message.type
-    assert message.state == 'waiting', message.state
-    sepa_string = message.message.encode('utf-8')
-    sepa_xml = etree.fromstring(sepa_string)
-    schema_file = os.path.join(os.path.dirname(__file__),
-        '%s.xsd' % xsd)
-    schema = etree.XMLSchema(etree.parse(schema_file))
-    schema.assertValid(sepa_xml)
+        session_id, _, _ = ProcessPayment.create()
+        process_payment = ProcessPayment(session_id)
+        with Transaction().set_context(active_ids=[payment.id]):
+            _, data = process_payment.do_process(None)
+        group, = PaymentGroup.browse(data['res_id'])
+        message, = group.sepa_messages
+        assert message.type == 'out', message.type
+        assert message.state == 'waiting', message.state
+        sepa_string = message.message.encode('utf-8')
+        sepa_xml = etree.fromstring(sepa_string)
+        schema_file = os.path.join(os.path.dirname(__file__),
+            '%s.xsd' % xsd)
+        schema = etree.XMLSchema(etree.parse(schema_file))
+        schema.assertValid(sepa_xml)
 
 
 class AccountPaymentSepaTestCase(ModuleTestCase):
     'Test Account Payment SEPA module'
     module = 'account_payment_sepa'
 
+    @with_transaction()
     def test_pain001_001_03(self):
         'Test pain001.001.03 xsd validation'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            validate_file('pain.001.001.03', 'payable')
+        validate_file('pain.001.001.03', 'payable')
 
+    @with_transaction()
     def test_pain001_001_05(self):
         'Test pain001.001.05 xsd validation'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            validate_file('pain.001.001.05', 'payable')
+        validate_file('pain.001.001.05', 'payable')
 
+    @with_transaction()
     def test_pain001_003_03(self):
         'Test pain001.003.03 xsd validation'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            validate_file('pain.001.003.03', 'payable')
+        validate_file('pain.001.003.03', 'payable')
 
+    @with_transaction()
     def test_pain008_001_02(self):
         'Test pain008.001.02 xsd validation'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            validate_file('pain.008.001.02', 'receivable')
+        validate_file('pain.008.001.02', 'receivable')
 
+    @with_transaction()
     def test_pain008_001_04(self):
         'Test pain008.001.04 xsd validation'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            validate_file('pain.008.001.04', 'receivable')
+        validate_file('pain.008.001.04', 'receivable')
 
+    @with_transaction()
     def test_pain008_003_02(self):
         'Test pain008.003.02 xsd validation'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            validate_file('pain.008.003.02', 'receivable')
+        validate_file('pain.008.003.02', 'receivable')
 
+    @with_transaction()
     def test_sepa_mandate_sequence(self):
         'Test SEPA mandate sequence'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            pool = Pool()
-            Configuration = pool.get('account.configuration')
-            Sequence = pool.get('ir.sequence')
-            Party = pool.get('party.party')
-            Mandate = pool.get('account.payment.sepa.mandate')
+        pool = Pool()
+        Configuration = pool.get('account.configuration')
+        Sequence = pool.get('ir.sequence')
+        Party = pool.get('party.party')
+        Mandate = pool.get('account.payment.sepa.mandate')
 
-            party = Party(name='Test')
-            party.save()
+        party = Party(name='Test')
+        party.save()
+
+        company = create_company()
+        with set_company(company):
             mandate = Mandate(party=party)
             mandate.save()
             self.assertFalse(mandate.identification)
@@ -213,17 +211,20 @@ class AccountPaymentSepaTestCase(ModuleTestCase):
             mandate.save()
             self.assertTrue(mandate.identification)
 
+    @with_transaction()
     def test_identification_unique(self):
         'Test unique identification constraint'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            pool = Pool()
-            Party = pool.get('party.party')
-            Mandate = pool.get('account.payment.sepa.mandate')
+        pool = Pool()
+        Party = pool.get('party.party')
+        Mandate = pool.get('account.payment.sepa.mandate')
 
-            same_id = '1'
+        same_id = '1'
 
-            party = Party(name='Test')
-            party.save()
+        party = Party(name='Test')
+        party.save()
+
+        company = create_company()
+        with set_company(company):
             mandate = Mandate(party=party, identification=same_id)
             mandate.save()
 
@@ -245,16 +246,19 @@ class AccountPaymentSepaTestCase(ModuleTestCase):
                         'identification': same_id,
                         }])
 
+    @with_transaction()
     def test_payment_sepa_bank_account_number(self):
         'Test Payment.sepa_bank_account_number'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            pool = Pool()
-            Payment = pool.get('account.payment')
-            Mandate = pool.get('account.payment.sepa.mandate')
-            AccountNumber = pool.get('bank.account.number')
-            Party = pool.get('party.party')
-            BankAccount = pool.get('bank.account')
+        pool = Pool()
+        Payment = pool.get('account.payment')
+        Mandate = pool.get('account.payment.sepa.mandate')
+        AccountNumber = pool.get('bank.account.number')
+        Party = pool.get('party.party')
+        BankAccount = pool.get('bank.account')
 
+        company = create_company()
+        with set_company(company):
+            create_chart(company)
             account_number = AccountNumber()
             mandate = Mandate(account_number=account_number)
             payment = Payment(kind='receivable', sepa_mandate=mandate)
@@ -273,18 +277,19 @@ class AccountPaymentSepaTestCase(ModuleTestCase):
             self.assertEqual(id(payment.sepa_bank_account_number),
                 id(iban_account_number))
 
+    @with_transaction()
     def test_payment_sequence_type(self):
         'Test payment sequence type'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            pool = Pool()
-            Date = pool.get('ir.date')
-            Payment = pool.get('account.payment')
-            ProcessPayment = pool.get('account.payment.process', type='wizard')
+        pool = Pool()
+        Date = pool.get('ir.date')
+        Payment = pool.get('account.payment')
+        ProcessPayment = pool.get('account.payment.process', type='wizard')
 
-            environment = setup_environment()
-            company = environment['company']
-            bank = environment['bank']
-            customer = environment['customer']
+        environment = setup_environment()
+        company = environment['company']
+        bank = environment['bank']
+        customer = environment['customer']
+        with set_company(company):
             company_account, customer_account = setup_accounts(
                 bank, company, customer)
             setup_mandate(company, customer, customer_account)
@@ -339,40 +344,40 @@ class AccountPaymentSepaTestCase(ModuleTestCase):
             for payment in payments:
                 self.assertEqual(payment.sepa_mandate_sequence_type, 'RCUR')
 
+    @with_transaction()
     def handle_camt054(self, flavor):
         'Handle camt.054'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            pool = Pool()
-            Message = pool.get('account.payment.sepa.message')
+        pool = Pool()
+        Message = pool.get('account.payment.sepa.message')
 
-            message_file = os.path.join(os.path.dirname(__file__),
-                '%s.xml' % flavor)
-            message = open(message_file).read()
-            namespace = Message.get_namespace(message)
-            self.assertEqual(namespace,
-                'urn:iso:std:iso:20022:tech:xsd:%s' % flavor)
+        message_file = os.path.join(os.path.dirname(__file__),
+            '%s.xml' % flavor)
+        message = open(message_file, 'rb').read()
+        namespace = Message.get_namespace(message)
+        self.assertEqual(namespace,
+            'urn:iso:std:iso:20022:tech:xsd:%s' % flavor)
 
-            payment = Mock()
-            Payment = Mock()
-            Payment.search.return_value = [payment]
+        payment = Mock()
+        Payment = Mock()
+        Payment.search.return_value = [payment]
 
+        handler = CAMT054(BytesIO(message), Payment)
+
+        self.assertEqual(handler.msg_id, 'AAAASESS-FP-00001')
+        Payment.search.assert_called_with([
+                ('sepa_end_to_end_id', '=', 'MUELL/FINP/RA12345'),
+                ('kind', '=', 'payable'),
+                ])
+        Payment.succeed.assert_called_with([payment])
+
+        payment.reset_mock()
+        Payment.reset_mock()
+        with patch.object(CAMT054, 'is_returned') as is_returned:
+            is_returned.return_value = True
             handler = CAMT054(BytesIO(message), Payment)
 
-            self.assertEqual(handler.msg_id, 'AAAASESS-FP-00001')
-            Payment.search.assert_called_with([
-                    ('sepa_end_to_end_id', '=', 'MUELL/FINP/RA12345'),
-                    ('kind', '=', 'payable'),
-                    ])
-            Payment.succeed.assert_called_with([payment])
-
-            payment.reset_mock()
-            Payment.reset_mock()
-            with patch.object(CAMT054, 'is_returned') as is_returned:
-                is_returned.return_value = True
-                handler = CAMT054(BytesIO(message), Payment)
-
-                Payment.save.assert_called_with([payment])
-                Payment.fail.assert_called_with([payment])
+            Payment.save.assert_called_with([payment])
+            Payment.fail.assert_called_with([payment])
 
     def test_camt054_001_01(self):
         'Test camt.054.001.01 handling'
@@ -390,16 +395,17 @@ class AccountPaymentSepaTestCase(ModuleTestCase):
         'Test camt.054.001.04 handling'
         self.handle_camt054('camt.054.001.04')
 
+    @with_transaction()
     def test_sepa_mandate_report(self):
         'Test sepa mandate report'
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
-            pool = Pool()
-            Report = pool.get('account.payment.sepa.mandate', type='report')
+        pool = Pool()
+        Report = pool.get('account.payment.sepa.mandate', type='report')
 
-            environment = setup_environment()
-            company = environment['company']
-            bank = environment['bank']
-            customer = environment['customer']
+        environment = setup_environment()
+        company = environment['company']
+        bank = environment['bank']
+        customer = environment['customer']
+        with set_company(company):
             company_account, customer_account = setup_accounts(
                 bank, company, customer)
             mandate = setup_mandate(company, customer, customer_account)
@@ -411,11 +417,6 @@ class AccountPaymentSepaTestCase(ModuleTestCase):
 
 def suite():
     suite = trytond.tests.test_tryton.suite()
-    from trytond.modules.company.tests import test_company
-    from trytond.modules.account.tests import test_account
-    for test in chain(test_company.suite(), test_account.suite()):
-        if test not in suite and not isinstance(test, doctest.DocTestCase):
-            suite.addTest(test)
     suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
             AccountPaymentSepaTestCase))
     return suite
